//
//  TargetView.m
//  AnimationExplorer
//
//  Created by Nathan Corvino on 8/15/12.
//
//

#import "TargetView.h"

#define TILE_LAYER_SIZE (100000.)
#define TILE_LAYER_ORIGIN (TILE_LAYER_SIZE / - 2.)
#define HASH_SPACING (50)
#define HASH_SIZE (3.)

typedef enum {
    BCGreen = 0,
    BCBlack,
    BCYellow,
    BCOrange,
    BCCyan,
    BCMagenta,
    BCPurple,
    BCBrown,
    BCNumberOfColors
} BoundsColors;

static CGColorRef colorForHash(CGPoint hash) {
    int colorNumber = (((int) hash.x + 2 * (int) hash.y) / HASH_SPACING) % BCNumberOfColors;
    UIColor *color;

    switch (colorNumber) {
        case BCGreen:
            color = [UIColor greenColor];
            break;
        case BCBlack:
            color = [UIColor blackColor];
            break;
        case BCYellow:
            color = [UIColor yellowColor];
            break;
        case BCOrange:
            color = [UIColor orangeColor];
            break;
        case BCCyan:
            color = [UIColor cyanColor];
            break;
        case BCMagenta:
            color = [UIColor magentaColor];
            break;
        case BCPurple:
            color = [UIColor purpleColor];
            break;
        case BCBrown:
            color = [UIColor brownColor];
            break;
        default:
            color = [UIColor whiteColor];
            break;
    }

    return color.CGColor;
}

@interface BoundsTiledLayerDelegate : NSObject
@end

@implementation BoundsTiledLayerDelegate

- (void)drawLayer:(CALayer*)layer inContext:(CGContextRef)context
{
    CGRect box = CGContextGetClipBoundingBox(context);

    int startX = ((int) box.origin.x / HASH_SPACING) * HASH_SPACING;
    int startY = ((int) box.origin.y / HASH_SPACING) * HASH_SPACING;

    CGContextSetTextMatrix(context, CGAffineTransformMakeScale(1., -1.));
    CGContextSelectFont(context, "Helvetica", 10., kCGEncodingMacRoman);

    for (int x = startX; x < box.origin.x + box.size.width; x += HASH_SPACING) {
        for (int y = startY; y < box.origin.y + box.size.height; y += HASH_SPACING) {
            CGPoint hash = CGPointMake(x, y);
            CGColorRef color = colorForHash(hash);
            CGPoint points[] = { CGPointMake(hash.x - HASH_SIZE, hash.y), CGPointMake(hash.x + HASH_SIZE, hash.y),
                                 CGPointMake(hash.x, hash.y - HASH_SIZE), CGPointMake(hash.x, hash.y + HASH_SIZE)};
            CGContextSetStrokeColorWithColor(context, color);
            CGContextStrokeLineSegments(context, points, 4);

            CGContextSetFillColorWithColor(context, color);

            const char *xText = ((NSString *) [NSString stringWithFormat:@"(%0.0f,", hash.x + TILE_LAYER_ORIGIN]).UTF8String;
            const char *yText = ((NSString *) [NSString stringWithFormat:@" %0.0f)", hash.y + TILE_LAYER_ORIGIN]).UTF8String;

            // Text positioning is a touch wonky; the magic values on y get the text to show up
            // below the hash, as the text is drawn above the point. Translating and rotating
            // the ctm might be nicer.

            CGContextShowTextAtPoint(context, hash.x + 2., hash.y + 8., xText, strlen(xText));
            CGContextShowTextAtPoint(context, hash.x + 2., hash.y + 19., yText, strlen(yText));
        }
    }
}

@end


@implementation TargetView

- (void)awakeFromNib
{
    self.layer.masksToBounds = YES;

    tiledDelegate = [[BoundsTiledLayerDelegate alloc] init];
    tiledLayer = [CATiledLayer layer];
    tiledLayer.delegate = tiledDelegate;
    tiledLayer.frame = CGRectMake(TILE_LAYER_ORIGIN, TILE_LAYER_ORIGIN, TILE_LAYER_SIZE, TILE_LAYER_SIZE);
    tiledLayer.contentsScale = [[UIScreen mainScreen] scale];

    [self.layer addSublayer:tiledLayer];
    [tiledLayer setNeedsDisplay];
}

@end
