//
//  ViewController.m
//  AnimationExplorer
//
//  Copyright (c) 2012 Nathan Corvino.
//  Some rights reserved: http://opensource.org/licenses/MIT
//

#import "ViewController.h"
#import <QuartzCore/QuartzCore.h>
#import "CloseButton.h"

#define CLOSE_BUTTON_WIDTH 44.
#define CLOSE_BUTTON_HEIGHT 44.

@implementation ViewController

@synthesize parentView;
@synthesize targetView;
@synthesize editPane;
@synthesize propertySegments;
@synthesize dataTypeLabel;
@synthesize rectPane;
@synthesize pointPane;
@synthesize transformPane;
@synthesize transform3DPane;
@synthesize rectOriginX;
@synthesize rectOriginY;
@synthesize rectWidth;
@synthesize rectHeight;
@synthesize pointX;
@synthesize pointY;
@synthesize transformA;
@synthesize transformB;
@synthesize transformC;
@synthesize transformD;
@synthesize transformTx;
@synthesize transformTy;
@synthesize transformM11;
@synthesize transformM12;
@synthesize transformM13;
@synthesize transformM14;
@synthesize transformM21;
@synthesize transformM22;
@synthesize transformM23;
@synthesize transformM24;
@synthesize transformM31;
@synthesize transformM32;
@synthesize transformM33;
@synthesize transformM34;
@synthesize transformM41;
@synthesize transformM42;
@synthesize transformM43;
@synthesize transformM44;

#pragma mark -
#pragma mark Transform Pane Properties

@synthesize transformSegments;
@synthesize affineTransformPopup;
@synthesize affineTransformEditPane;
@synthesize rotatePane;
@synthesize scalePane;
@synthesize translatePane;
@synthesize concatPane;
@synthesize makeNewRotation;
@synthesize transformRotateValue;
@synthesize makeNewScale;
@synthesize transformSx;
@synthesize transformSy;
@synthesize makeNewTranslation;
@synthesize transformPaneTx;
@synthesize transformPaneTy;
@synthesize transformPaneA;
@synthesize transformPaneB;
@synthesize transformPaneC;
@synthesize transformPaneD;
@synthesize transformPaneConcatTx;
@synthesize transformPaneConcatTy;

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

- (void)showRectProperties:(CGRect)rect
{
    rectOriginX.text = [NSString stringWithFormat:@"%5.2f", rect.origin.x];
    rectOriginY.text = [NSString stringWithFormat:@"%5.2f", rect.origin.y];
    rectHeight.text = [NSString stringWithFormat:@"%5.2f", rect.size.height];
    rectWidth.text = [NSString stringWithFormat:@"%5.2f", rect.size.width];
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    currentPane = rectPane;
    [editPane addSubview:currentPane];
    [self showRectProperties:targetView.frame];
}

- (void)viewDidUnload
{
    [self setRectOriginX:nil];
    [self setRectOriginY:nil];
    [self setRectHeight:nil];
    [self setRectWidth:nil];
    [self setEditPane:nil];
    [self setPropertySegments:nil];
    [self setDataTypeLabel:nil];
    [self setPointX:nil];
    [self setPointY:nil];
    [self setRectPane:nil];
    [self setPointPane:nil];
    [self setTransformPane:nil];
    [self setTransform3DPane:nil];
    [self setTransformA:nil];
    [self setTransformB:nil];
    [self setTransformC:nil];
    [self setTransformD:nil];
    [self setTransformTx:nil];
    [self setTransformTy:nil];
    [self setTransformM11:nil];
    [self setTransformM12:nil];
    [self setTransformM13:nil];
    [self setTransformM13:nil];
    [self setTransformM21:nil];
    [self setTransformM22:nil];
    [self setTransformM23:nil];
    [self setTransformM24:nil];
    [self setTransformM31:nil];
    [self setTransformM32:nil];
    [self setTransformM33:nil];
    [self setTransformM34:nil];
    [self setTransformM41:nil];
    [self setTransformM42:nil];
    [self setTransformM43:nil];
    [self setTransformM44:nil];
    [self setParentView:nil];
    [self setAffineTransformPopup:nil];
    [self setMakeNewRotation:nil];
    [self setTransformRotateValue:nil];
    [self setTransformSegments:nil];
    [self setRotatePane:nil];
    [self setScalePane:nil];
    [self setConcatPane:nil];
    [self setMakeNewScale:nil];
    [self setTransformSx:nil];
    [self setTransformSy:nil];
    [self setMakeNewTranslation:nil];
    [self setTransformTx:nil];
    [self setTransformTy:nil];
    [self setAffineTransformEditPane:nil];
    [self setTransformPaneA:nil];
    [self setTransformPaneB:nil];
    [self setTransformPaneC:nil];
    [self setTransformPaneD:nil];
    [self setTransformPaneConcatTx:nil];
    [self setTransformPaneConcatTy:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
    [super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
    if (0 == propertySegments.selectedSegmentIndex) {
        [self showRectProperties:targetView.frame];
    } else if (2 == propertySegments.selectedSegmentIndex) {
        pointX.text = [NSString stringWithFormat:@"%5.2f", targetView.center.x];
        pointY.text = [NSString stringWithFormat:@"%5.2f", targetView.center.y];
    }
}

- (IBAction)segmentChanged:(UISegmentedControl *)sender;
{
    [UIView animateWithDuration:0.3 animations:^{
        editPane.center = CGPointMake(editPane.center.x, editPane.center.y - editPane.frame.size.height);
        propertySegments.enabled = NO;
    } completion:^(BOOL finished) {
        [currentPane removeFromSuperview];

        switch (propertySegments.selectedSegmentIndex) {
            case 0:
                [self showRectProperties:targetView.frame];
                currentPane = rectPane;
                break;

            case 1:
                [self showRectProperties:targetView.bounds];
                currentPane = rectPane;
                break;

            case 2:
                pointX.text = [NSString stringWithFormat:@"%5.2f", targetView.center.x];
                pointY.text = [NSString stringWithFormat:@"%5.2f", targetView.center.y];
                currentPane = pointPane;
                break;

            case 3:
                transformA.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.a];
                transformB.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.b];
                transformC.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.c];
                transformD.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.d];
                transformTx.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.ty];
                transformTy.text = [NSString stringWithFormat:@"%5.2f", targetView.transform.tx];
                currentPane = transformPane;
                break;

            case 4:
                transformM11.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m11];
                transformM12.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m12];
                transformM13.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m13];
                transformM14.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m14];

                transformM21.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m21];
                transformM22.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m22];
                transformM23.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m23];
                transformM24.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m24];

                transformM31.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m31];
                transformM32.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m32];
                transformM33.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m33];
                transformM34.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m34];

                transformM41.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m41];
                transformM42.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m42];
                transformM43.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m43];
                transformM44.text = [NSString stringWithFormat:@"%5.2f", targetView.layer.transform.m44];

                currentPane = transform3DPane;
                break;

            default:
                currentPane = nil;
                NSAssert(FALSE, @"Changed to segement that shouldn't exist!");
                break;
        }

        currentPane.frame = CGRectMake(0., 0., editPane.frame.size.width, editPane.frame.size.height);
        [editPane addSubview:currentPane];
        [UIView animateWithDuration:0.3 animations:^{
            editPane.center = CGPointMake(editPane.center.x, editPane.center.y + editPane.frame.size.height);
        } completion:^(BOOL finished) {
            propertySegments.enabled = YES;
        }];
    }];
}

- (IBAction)changeProperty:(id)sender {
    CGPathRef oldPath = CGPathRetain(parentView.frameShape.path);

    [UIView animateWithDuration:0.8 animations:^{
        switch ([propertySegments selectedSegmentIndex]) {
            case 0:
                targetView.frame = CGRectMake([rectOriginX.text floatValue], [rectOriginY.text floatValue], [rectWidth.text floatValue], [rectHeight.text floatValue]);
                break;

            case 1:
                targetView.bounds = CGRectMake([rectOriginX.text floatValue], [rectOriginY.text floatValue], [rectWidth.text floatValue], [rectHeight.text floatValue]);
                break;

            case 2:
                targetView.center = CGPointMake([pointX.text floatValue], [pointY.text floatValue]);
                break;

            case 3:
                targetView.transform = CGAffineTransformMake([transformA.text floatValue], [transformB.text floatValue], [transformC.text floatValue], [transformD.text floatValue], [transformTx.text floatValue], [transformTy.text floatValue]);
                break;

            case 4: {
                CATransform3D transform;

                transform.m11 = [transformM11.text floatValue];
                transform.m12 = [transformM12.text floatValue];
                transform.m13 = [transformM13.text floatValue];
                transform.m14 = [transformM14.text floatValue];

                transform.m21 = [transformM21.text floatValue];
                transform.m22 = [transformM22.text floatValue];
                transform.m23 = [transformM23.text floatValue];
                transform.m24 = [transformM24.text floatValue];

                transform.m31 = [transformM31.text floatValue];
                transform.m32 = [transformM32.text floatValue];
                transform.m33 = [transformM33.text floatValue];
                transform.m34 = [transformM34.text floatValue];

                transform.m41 = [transformM41.text floatValue];
                transform.m42 = [transformM42.text floatValue];
                transform.m43 = [transformM43.text floatValue];
                transform.m44 = [transformM44.text floatValue];

                targetView.layer.transform = transform;
                break;
            }

            default:
                NSAssert(FALSE, @"Set for segment that shouldn't exist!");
                break;
        }

        CGPathRef newPath = parentView.frameShape.path;

        if (CGPathEqualToPath(oldPath, newPath)) {
            CGPathRelease(oldPath);
        } else {
            CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"path"];
            animation.fromValue = (__bridge_transfer id) oldPath;
            animation.toValue = (__bridge_transfer id) CGPathRetain(parentView.frameShape.path);
            animation.duration = 0.8;
            [parentView.frameShape addAnimation:animation forKey:@"path"];
        }
    }];
}

- (void)closePopup:(id)sender
{
    [UIView animateWithDuration:0.2 animations:^{
        popupContent.transform = CGAffineTransformMakeScale(0.001, 0.001);
    } completion:^(BOOL finished) {
        [popupBackground removeFromSuperview];
        popupBackground = nil;
        popupContent = nil;
    }];
}

- (void)showPanel:(UIView *)panel
{
    CGRect frame = self.view.bounds;
    CGPoint center = CGPointMake(frame.size.width / 2., panel.frame.size.height - 22.);
    UIView* blankBack = [[UIView alloc] initWithFrame:CGRectMake(0., 0., 644., 644.)];
    blankBack.center = center;
    blankBack.backgroundColor = [UIColor orangeColor];

    [popupBackground addSubview:blankBack];

    CGSize paneSize = panel.frame.size;
    popupContent = [[UIView alloc] initWithFrame:CGRectMake(0., 0., paneSize.width + CLOSE_BUTTON_WIDTH, paneSize.height + CLOSE_BUTTON_HEIGHT)];
    popupContent.center = center;
    popupContent.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin;

    panel.frame = CGRectMake(CLOSE_BUTTON_WIDTH / 2., CLOSE_BUTTON_HEIGHT / 2., paneSize.width, paneSize.height);
    panel.layer.shadowPath = CGPathCreateWithRect(panel.bounds, NULL);
    CGPathRelease(panel.layer.shadowPath);
    panel.layer.shadowOpacity = 0.8;
    [popupContent addSubview:panel];

    CloseButton *closeButton = [[CloseButton alloc] initWithFrame:CGRectMake(0., 0., CLOSE_BUTTON_WIDTH, CLOSE_BUTTON_HEIGHT)];
    closeButton.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    [closeButton addTarget:self action:@selector(closePopup:) forControlEvents:UIControlEventTouchUpInside];
    [popupContent addSubview:closeButton];

    [UIView transitionFromView:blankBack
                        toView:popupContent
                      duration:0.4
                       options:UIViewAnimationOptionTransitionFlipFromRight | UIViewAnimationOptionAllowUserInteraction | UIViewAnimationOptionBeginFromCurrentState
                    completion:^(BOOL finished) {

                    }];
}

- (IBAction)showOptions:(id)sender {
    popupBackground = [[UIView alloc] initWithFrame:CGRectMake(0., 0., self.view.bounds.size.width, self.view.bounds.size.height)];
    popupBackground.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

    NSArray *nibObjects = [[NSBundle mainBundle] loadNibNamed:@"AffineTransform" owner:self options:nil];
    NSAssert(nibObjects, @"Failed to load AffineTransform nib");

    makeNewRotation.on = NO;
    makeNewScale.on = NO;
    makeNewTranslation.on = NO;
    [affineTransformEditPane addSubview:rotatePane];

    [self.view addSubview:popupBackground];
    [self performSelector:@selector(showPanel:) withObject:affineTransformPopup afterDelay:1e-100];
}

#pragma mark - Transform Pane

- (IBAction)transformOperationChanged:(id)sender {
    [currentPopupPane removeFromSuperview];

    switch ([transformSegments selectedSegmentIndex]) {
        case 0:
            currentPopupPane = rotatePane;
            break;
        case 1:
            currentPopupPane = scalePane;
            break;
        case 2:
            currentPopupPane = translatePane;
            break;
        case 3:
            currentPopupPane = concatPane;
            break;
        default:
            break;
    }

    [affineTransformEditPane addSubview:currentPopupPane];
}

- (IBAction)doTransform:(id)sender
{
    switch ([transformSegments selectedSegmentIndex]) {
        case 0:
            if (makeNewRotation.on) {
                targetView.transform = CGAffineTransformMakeRotation([transformRotateValue.text floatValue]);
            } else {
                targetView.transform = CGAffineTransformRotate(targetView.transform, [transformRotateValue.text floatValue]);
            }
            break;

        case 1:
            if (makeNewScale.on) {
                targetView.transform = CGAffineTransformMakeScale([transformSx.text floatValue], [transformSy.text floatValue]);
            } else {
                targetView.transform = CGAffineTransformScale(targetView.transform, [transformSx.text floatValue], [transformSy.text floatValue]);
            }
            break;

        case 2:
            if (makeNewScale.on) {
                targetView.transform = CGAffineTransformMakeTranslation([transformTx.text floatValue], [transformTy.text floatValue]);
            } else {
                targetView.transform = CGAffineTransformTranslate(targetView.transform, [transformTx.text floatValue], [transformTy.text floatValue]);
            }
            break;

        case 3:
            targetView.transform =
            CGAffineTransformConcat(targetView.transform,
                                    CGAffineTransformMake([transformPaneA.text floatValue],
                                                          [transformPaneB.text floatValue],
                                                          [transformPaneC.text floatValue],
                                                          [transformPaneD.text floatValue],
                                                          [transformPaneConcatTx.text floatValue],
                                                          [transformPaneConcatTy.text floatValue]));
            break;
    }
}


@end
